//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * IC[]zԎ
 * @author Junpee
 */

#include "LampBasic.h"
#include "Animation/RotationInterpolator/EulerArrayInterpolator.h"
#include "Animation/RotationInterpolator/QuaternionArrayInterpolator.h"

namespace Lamp{

//------------------------------------------------------------------------------
// RXgN^
EulerArrayInterpolator::EulerArrayInterpolator() :
	array_(NULL), size_(0), length_(0.f){
}
//------------------------------------------------------------------------------
// fXgN^
EulerArrayInterpolator::~EulerArrayInterpolator(){
	SafeArrayDelete(array_);
}
//------------------------------------------------------------------------------
// Rs[RXgN^
EulerArrayInterpolator::EulerArrayInterpolator(
	const EulerArrayInterpolator& copy){
	size_ = copy.size_;
	length_ = copy.length_;
	// Rs[RXgN^͋INULLݒ
	array_ = NULL;
	if(size_ == 0){ return; }
	array_ = new Vector3[size_];
	std::memcpy(array_, copy.array_, sizeof(Vector3) * size_);
}
//------------------------------------------------------------------------------
// Zq
EulerArrayInterpolator& EulerArrayInterpolator::operator =(
	const EulerArrayInterpolator& copy){
	// gȂ烊^[
	if(this == &copy){ return *this; }
	size_ = copy.size_;
	length_ = copy.length_;
	// ̍ۂ͉NULLݒ
	SafeArrayDelete(array_);
	if(size_ == 0){ return *this; }
	array_ = new Vector3[size_];
	std::memcpy(array_, copy.array_, sizeof(Vector3) * size_);
	return *this;
}
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// IC[
Vector3 EulerArrayInterpolator::eulerInterpolate(float time){
	Assert(array_ != NULL);
	// ԂOɂӂĂ
	if(time <= 0.f){ return array_[0]; }
	// ԂɂӂĂ
	if(time >= length_){ return array_[size_ - 1]; }
	// lŕԂăIC[pɕϊ
	Quaternion resultQuaternion = quaternionInterpolate(time);
	Vector3 result;
	resultQuaternion.getRotationXYZ(&result);
#ifdef _DEBUG
	if((!Math::classCheck(result.x)) ||
		(!Math::classCheck(result.y)) ||
		(!Math::classCheck(result.z))){
		_asm{ int 3 }
	}
#endif
	return result;
}
//------------------------------------------------------------------------------
// l
Quaternion EulerArrayInterpolator::quaternionInterpolate(float time){
	Assert(array_ != NULL);
	Quaternion result;
	// ԂOɂӂĂ
	if(time <= 0.f){
		result.setRotationXYZ(array_[0]);
		return result;
	}
	// ԂɂӂĂ
	if(time >= length_){
		result.setRotationXYZ(array_[size_ - 1]);
		return result;
	}
	// 
	float integer, rate;
	rate = Math::modf(time, &integer);
	int index = (int)integer;
	// l
	Quaternion pre, post;
	pre.setRotationXYZ(array_[index]);
	post.setRotationXYZ(array_[index + 1]);
	// ␳t`Ԃs
	result = Quaternion::correctSlerp(pre, post, rate);
#ifdef _DEBUG
	if((!Math::classCheck(result.x)) ||
		(!Math::classCheck(result.y)) ||
		(!Math::classCheck(result.z)) ||
		(!Math::classCheck(result.w))){
		_asm{ int 3 }
	}
#endif
	return result;
}
//------------------------------------------------------------------------------
// l
//------------------------------------------------------------------------------
// TCYݒ
void EulerArrayInterpolator::setSize(int size){
	// Ԃׂ̈ɂ2ȏsizeKv
	Assert(size > 1);
	size_ = size;
	length_ = (float)(size_ - 1);
	SafeArrayDelete(array_);
	array_ = new Vector3[size_];
}
//------------------------------------------------------------------------------
// l̐ݒ
void EulerArrayInterpolator::setValue(int index, const Vector3& value){
	Assert(array_ != NULL);
	Assert(index >= 0);
	Assert(index < size_);
	// +-PIɃNv
	Vector3 correctValue;
	for(int i = 0; i < 3; i++){
		float temp = value.array[i];
		if((temp <= Math::PI) && (temp >= -Math::PI)){
			correctValue.array[i] = temp;
			continue;
		}
		temp += Math::PI;
		if(temp < 0.f){
			temp = Math::abs(temp);
			temp = Math::fmod(temp, Math::doublePI) - Math::PI;
			temp *= -1;
		}else{
			temp = Math::fmod(temp, Math::doublePI) - Math::PI;
		}
		correctValue.array[i] = temp;
	}
	array_[index] = correctValue;
}
//------------------------------------------------------------------------------
// ϊ
//------------------------------------------------------------------------------
// l]zԂւ̕ϊ
QuaternionArrayInterpolator*
	EulerArrayInterpolator::convertQuaternionArrayInterpolator() const{
	QuaternionArrayInterpolator* result = new QuaternionArrayInterpolator();
	int size = getSize();
	result->setSize(size);
	Quaternion quaternion;
	for(int i = 0; i < size; i++){
		quaternion.setRotationXYZ(getValue(i));
		result->setValue(i, quaternion);
	}
	// l␳Ă
	result->correctValue();
	return result;
}
//------------------------------------------------------------------------------
} // End of namespace Lamp
//------------------------------------------------------------------------------
